@@ -1,6 +1,6 @@
 package Data::RuledValidator;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use strict;
 use warnings "all";
@@ -12,14 +12,46 @@ use overload
   '@{}' => \&_result;
 
 my %RULES;
-my %RULE_ID_KEY;
-
-# condition operator
 my %COND_OP;
-
-# create closure
 my %MK_CLOSURE;
 
+sub _rules{
+  my($self, $rule) = @_;
+  if(@_ == 2){
+    if(exists $RULES{$rule}){
+      my $t = (stat $rule)[9];
+      $RULES{$rule}->{time} ||= $t;
+      if($RULES{$rule}->{time} < $t){
+        delete $RULES{$rule}->{coded_rule};
+        $RULES{$rule}->{time} = $t;
+      }
+    }
+    return $RULES{$rule}->{coded_rule} ||= {};
+  }else{
+    return \%RULES;
+  }
+}
+
+sub id_key{
+  my($self, $rule, $id_key) = @_;
+  @_ == 3 ? $RULES{$rule}->{id_key} = $id_key: $RULES{$rule}->{id_key};
+};
+
+sub id_method{
+  my($self, $rule, $id_method) = @_;
+  $rule ||= $self->rule or Carp::croak("first argument 'rule' is missing;");
+  @_ == 3 ? $RULES{$rule}->{id_method} = $id_method: $RULES{$rule}->{id_method} || $self->{id_method};
+};
+
+sub _regex_group{
+  my($self, $rule, $id_name) = @_;
+  if(@_ == 3){
+    return push @{$RULES{$rule}->{_regex_group} ||= []}, $id_name;
+  }else{
+    return @{$RULES{$rule}->{_regex_group} || []}
+  }
+};
+
 sub import{
   my($class, %option) = @_;
   foreach my $plugin (__PACKAGE__->plugins){
@@ -49,7 +81,7 @@ sub add_operator{
 sub add_condition_operator{
   my($self, %op_sub) = @_;
   while(my($op, $sub) = each %op_sub){
-    if($COND_OP{$op}++){
+    if(defined $COND_OP{$op}){
       Carp::croak("$op is alredy defined as condition operator.");
     }
     $COND_OP{$op} = $sub;
@@ -76,13 +108,23 @@ sub create_alias_cond_operator{
 
 sub new{
   my($class, %option) = @_;
-  $option{result} = {};
-  $option{valid}  = 0;
-  $option{rule} ||= '';
+  $option{result}   = {};
+  $option{valid}    = 0;
+  $option{strict} ||= 0;
+  $option{rule}   ||= '';
   my $o = bless \%option, $class;
   return $o;
 }
 
+sub rule{
+  my($self, $rule) = @_;
+  if(@_ == 2){
+    return $self->{rule} = $rule;
+  }else{
+    return $self->{rule};
+  }
+}
+
 sub list_plugins{
   my($self) = @_;
   return $self->plugins;
@@ -90,19 +132,19 @@ sub list_plugins{
 
 sub obj{ shift()->{obj} };
 
+sub id_obj{ shift()->{id_obj} };
+
 sub to_obj{ shift()->{to_obj} };
 
 sub method{ shift()->{method} };
 
-sub id_key{ my($self, $rule) = @_; $RULE_ID_KEY{$rule} };
-
 sub _parse_definition{
   my($self, $defs) = @_;
   my @def;
   foreach my $def (@$defs){
     my $alias = $def =~ s/^\s*(\w+)\s*=\s*// ? $1 : '';
-    my($key, $op, $cond) = split /[\s\-]+/, $def, 3;
-    my $closure = $MK_CLOSURE{$op} ? $MK_CLOSURE{$op}->($key, $cond) : Carp::croak("not defined operator: $op");
+    my($key, $op, $cond) = split /\s+/, $def, 3;
+    my $closure = $MK_CLOSURE{$op} ? $MK_CLOSURE{$op}->($key, $cond, $op) : Carp::croak("not defined operator: $op");
     push @def, [$alias, $key, $op, $closure];
   }
   return \@def;
@@ -115,31 +157,44 @@ sub by_sentence{
   return $self->_validator($defs);
 }
 
+sub _get_value{
+  my($self, $last_obj, $method, $key) = @_;
+  my @method =  ref $method ? @$method : $method;
+  $last_obj = $last_obj->$_ foreach (@method[0 .. ($#method - 1)]);
+  my $m = $method[$#method];
+  return $last_obj->$m($key);
+}
+
 sub _validator{
   my($self, $defs) = @_;
   my $obj = $self->obj;
   my $method = $self->method;
-  my %result;
+  my(%result, %failure);
   my $result;
   my $all_result = 1;
   my($def_num, $ok_num);
+  $self->{result} = \%result;
+  $self->{failure}  = \%failure;
 
   foreach my $def (@$defs){
     my($alias, $key, $op, $sub) = @$def;
+    Carp::craok('cannot define same combination of key and operator twice.') if exists $result{$alias . '_' . $op};
     $alias ||= $key;
-    my @value = $obj->$method($key);
+    my @value = $self->_get_value($obj, $method, $key);
     if(defined $MK_CLOSURE{$op}){
       $result = $sub->($self, \@value, $alias, $obj, $method) || 0;
-      $result{$alias . '_' . $op} = $result;
-      $result{$alias . '_valid'} = 1 unless defined $result{$alias . '_valid'};
-      $result{$alias . '_valid'} &= $result;
+      unless($result{$alias . '_' . $op} = $result){
+        $failure{$alias . '_' . $op} = \@value unless not @value or defined $failure{$alias . '_' . $op};
+      }else{
+        $result{$alias . '_valid'} = 1 unless defined $result{$alias . '_valid'};
+        $result{$alias . '_valid'} &= $result;
+      }
     }
     $all_result &= $result;
   }
 
   # $result &= (($ok_num || 0) == $def_num + 0);
-  $self->{result} = \%result;
-  $self->{valid} = $all_result;
+  $self->{valid}  = $all_result;
   return $self;
 }
 
@@ -148,6 +203,11 @@ sub result{
   return($self->{result} || {});
 }
 
+sub failure{
+  my($self) = @_;
+  return($self->{failure} || {});
+}
+
 sub _result{
   my($self) = @_;
   return [%{$self->{result}}];
@@ -155,28 +215,32 @@ sub _result{
 
 sub valid{
   my($self) = @_;
-  return $self->{valid};
+  # too ugly
+  return $self->{strict} ? $self->{valid} : exists $self->{failure} ? %{$self->failure} ? 0 : 1 : 0;
 }
 
 sub reset{
   my($self) = @_;
-  delete $self->{$_} foreach qw/result valid/;
-}
-
-sub get_rule {
-  my($self, $rule, $rule_name) = @_;
-  return $RULES{$rule}->{$rule_name};
+  delete $self->{$_} foreach qw/result valid failure wrong right/;
 }
 
 sub by_rule{
   my($self, $rule) = @_;
-  $rule ||= $self->{rule};
+  $rule = $rule ? $self->rule($rule) : $self->rule;
   Carp::croak("need rule name")  unless $rule;
-  $self->_parse_rule($rule) unless defined $RULES{$rule};
-  my($obj, $method) = ($self->obj, $self->method);
+  $self->_parse_rule($rule) unless %{ $self->_rules($rule) || {} };
+  my($obj, $method) = ($self->id_obj || $self->obj, $self->id_method($rule) || $self->method);
   my $id_key = $self->id_key($rule);
-  my $rule_name = $id_key =~/^ENV_(.+)$/ ? $ENV{$1} : $obj->$method($id_key);
-  return $self->_validator($self->get_rule($rule, $rule_name) || []);
+  my $rule_name = $id_key =~ /^ENV_(.+)$/i ? $ENV{uc($1)} : $self->_get_value($obj, $method, $id_key || ());
+
+  my $defs = $self->_rules($rule)->{$rule_name};
+  unless($defs){
+    foreach my $r ($self->_regex_group($rule)){
+      last if $defs = $self->_rules($rule)->{$r};
+    }
+  }
+  $self->{rule_name} = $rule_name;
+  return $self->_validator($defs || []);
 }
 
 sub _distinct_rule{
@@ -199,7 +263,7 @@ sub _distinct_rule{
 
 sub _parse_rule{
   my($self, $rule) = @_;
-  my $rules = $RULES{$rule} ||= {};
+  my $rules = $self->_rules($rule);
   my $id_name = 'GLOBAL';
   my @rule;
   if(ref $rule eq 'SCALAR'){
@@ -215,13 +279,25 @@ sub _parse_rule{
     my $line = $_;
     $line =~s/^\s+//;
     $line =~s/\s+$//;
-    next unless $line or $line =~/^\s*#/;
-    if($line =~s/^ID_KEY\s+//){
-      $RULE_ID_KEY{$rule} = $line;
-    }elsif($line =~/^\s*\{\s*([\w\/]+)\s*\}\s*$/ or $line =~m|^\s*;+\s*([\w\/]+)\s*$|){
+    next unless $line and $line !~ /^\s*#/;
+    my $is_regex = 0;
+    if($line =~ s/^;+path;+/;/ or $line =~ s/path\{/\{/){
+      $is_regex = 1;
+      $line =~ s{/+$}{};
+      $line = '^'. $line . '/?$';
+    }elsif($line =~ s/^;+r;+/;/ or $line =~ s/r\{/\{/){
+      $is_regex = 1;
+    }
+    if($line =~s/^ID_KEY\s+//i){
+      $self->id_key($rule, $line);
+    }elsif($line =~s/^ID_METHOD\s+//i){
+      my @method = split /\s+/, $line;
+      $self->id_method($rule, [grep $_, @method]);
+    }elsif($line =~/^\s*\{\s*([^\s]+)\s*\}\s*$/ or $line =~m|^\s*;+\s*([^\s]+)\s*$|){
       # page name
       $id_name = $1;
       $rules->{$id_name} ||= [];
+      $self->_regex_group($rule, $id_name) if $is_regex;
     }else{
       # rule
       push @{$rules->{$id_name}}, $line;
@@ -238,6 +314,8 @@ sub _parse_rule{
 
 sub ok{ return shift()->{result}->{shift()}}
 
+sub valid_ok{ return shift()->{result}->{shift() . '_valid'}}
+
 use Data::RuledValidator::Closure;
 
 1;
@@ -349,7 +427,8 @@ Operator is operator to check Value(s).
 
 =item * Condition
 
-Condition is the condition for Operator to judge whether Value(s) is/are valid or not.
+Condition is the condition for Operator
+to judge whether Value(s) is/are valid or not.
 
 =back
 
@@ -389,9 +468,45 @@ $obj is Object which has values which you want to check.
 $method is Method of $obj which returns Value(s) which you want to check.
 $rule_file_location is file location of rule file.
 
- Data::RuledValidator->new(obj => $obj, method => $method);
+ my $v = Data::RuledValidator->new(obj => $obj, method => $method);
 
-If you use "by_sentence" and/or you use "by_rule" with argument, no need to use rule.
+If you use "by_sentence" and/or you use "by_rule" with argument, no need to specify rule here.
+
+You can use array ref for method. for example, $c is object, and $c->res->param is the way to get values.
+pass [qw/res param/] to method.
+
+If you need another object and/or method for identify to group name.
+
+ my $v = Data::RuledValidator->new(obj => $obj, method => $method, id_obj => $id_obj, id_method => $id_method);
+
+for validation, $obj->$method is used.
+for identifying to group name, $id_obj->$id_method is used (when you omit id_method, method is used).
+
+=back
+
+=head2 CONSTRUCTOR OPTION
+
+=over 4
+
+=item rule
+
+ rule => rule_file_location
+
+explained above.
+
+=item strict
+
+ strict => 0
+
+Before 0.06, Data::RuledValidator returns true with valid method,
+only when all key is valid.
+It means you need set "key is n/a",
+if you don't use keys defined in GLOABAL in other group.
+I think it is very bother.
+
+I will change it in future. At present, if strict is 0(it's default),
+if you have no wrong value, valid method returns true.
+But, if you reset, it returns undef.
 
 =back
 
@@ -401,10 +516,9 @@ If you use "by_sentence" and/or you use "by_rule" with argument, no need to use
 
 =item by_sentence
 
- $v->by_sentence("i is number");
+ $v->by_sentence("i is number", "k is word", ...);
 
-arguments is rule.
-You can write multiple sentence.
+The arguments is rule. You can write multiple sentence.
 It returnes $v object.
 
 =item by_rule
@@ -442,13 +556,48 @@ You can get this result as following:
 
  $v->valid;
 
-The result of validation check.
+The result of total validation check.
 The returned value is 1 or 0.
 
+If all rule is ok, valid is 1.
+If not, valid is 0.
+
 You can get this result as following:
 
  $result = $v;
 
+=item failure
+
+ $v->failure;
+
+Given values to validation check.
+Some/All of them are wrong value.
+This returned, for example, the following structure.
+
+ {
+   'i_is'    => ['x', 'y', 'z'],
+   'v_is'    => ['x@x.jp'],
+   'z_match' => [0123, 1234],
+ }
+
+If you want wrong value only, use wrong method.
+
+=item wrong
+
+This is not implemented.
+
+ $v->wrong;
+
+It returns only wrong value.
+
+ {
+   'i_is'    => ['x', 'y', 'z'],
+   'v_is'    => ['x@x.jp'],
+   'z_match' => [0123, 1234],
+ }
+
+All of them are wrong values.
+
 =item reset
 
  $v->reset();
@@ -457,6 +606,18 @@ The result of validation check is resetted.
 
 =back
 
+=head1 OTHER METHOD
+
+=over 4
+
+=item list_plugins
+
+ $v->list_plugins;
+
+list all plugins.
+
+=back
+
 =head1 RULE SYNTAX
 
 Rule Syntax is very simple.
@@ -466,23 +627,62 @@ Rule Syntax is very simple.
 =item ID_KEY Key
 
 The right value is key which is pased to Object->Method.
-The returned value of Object->Method(Key) is used to identify id_key_value.
+The returned value of Object->Method(Key) is used to identify GROUP_NAME
 
  ID_KEY page
 
-=item ;id_key_value
+=item ID_METHOD method method ...
+
+Note that: It is used, only when you need another method to identify to GROUP_NAME.
+
+The right value is method which is used when Object->Method.
+The returned value of Object->Method(Key)/Object->Method (Key is ommited)
+is used to identify GROUP_NAME.
+
+ ID_METHOD request action
+
+This can be defined in constructor, new.
+
+=item ;GROUP_NAME
 
 start from ; is start of group and the end of this group is the line before next ';'.
-If the value of Object->Method(ID_KEY) is equal id_key_value, this group validation rule is used.
+If the value of Object->Method(ID_KEY) is equal GROUP_NAME, this group validation rule is used.
 
  ;index
 
-=item ;r;^id_key_value$ (not yet implemented)
+You can write as following.
+
+ {index}
+ ;;;;index
+
+You can repeat ';' anytimes.
+
+=item ;r;^GROUP_NAME$
 
 This is start of group, too.
-If the value of Object->Method(ID_KEY) is match id_key_value, this group validation rule is used.
+If the value of Object->Method(ID_KEY) is match regexp ^GROUP_NAME$, this group validation rule is used.
 
- ;r; ^.*_confirm$
+ ;r;^.*_confirm$
+
+You can write as following.
+
+ r{^*_confirm$}
+ ;;r;;^.*_confirm$
+
+You can repeat ';' anytimes.
+
+=item ;path;/path/to/where
+
+It is as same as ;r;^/path/to/where/?$.
+
+Note that: this is needed that ID_KEY is 'ENV_PATH_INFO'.
+
+You can write as following.
+
+path{/path/to/where}
+;;path;;/path/to/where
+
+You can repeat ';' anytimes.
 
 =item ;GLOBAL
 
@@ -529,11 +729,13 @@ It is depends on Operator implementation.
 
 For example, Operator 'match' can multiple Condition.
 
- i match ^[a-z]+$ ^[0-9]+$
+ i match ^[a-z]+$,^[0-9]+$
+
+When i is match former or later, it is valid.
 
 Note that:
 
-You cannot use same operator for same key.
+You CANNOT use same key with same operator.
 
  i is number
  i is word
@@ -541,7 +743,7 @@ You cannot use same operator for same key.
 =item alias = sentence
 
 sentence is as same as above.
-alias = effects result data structure.
+'alias =' effects result data structure.
 
 First example is normal version.
 
@@ -608,10 +810,153 @@ If you want delete all GLOBAL rule in 'index' group.
 
 =over 4
 
-=item eq
+=item is
+
+ key is mail
+ key is word
+ key is number
+
+'is' is something special operator.
+It can be to be unavailable GLOBAL at all or about some key.
+
+ ;;GLOABAL
+ i is number
+ k is value
+
+ ;;index
+ v is word
+
+in this rule, 'index' inherits GLOBAL.
+If you want not to use GLOBAL.
+
+ ;;index
+ GLOBAL is n/a
+ v is word
+
+if you want not to use key 'k' in index.
+
+ ;;index
+ k is n/a
+ v is word
+
+This inherits 'i', but doesn't inherit 'k'.
+
+=item isnt
+
+It is the opposite of 'is'.
+but, no use to use 'n/a' in conidition.
+
+=item of
+
+This is some different from others.
+Left word is not key. number or 'all'.
+
+ all of x,y,z
+
+This is needed all of keys x, y and z.
+It is no need for these value of keys to be valid.
+If this key is exists, it is OK.
+
+If you need only 2 of these keys. you can write;
+
+ 2 of x,y,z
+
+This is needed 2 of keys x, y or z.
+
+If you want valid values, use of-valid instead of valid.
+
+=item of-valid
+
+This likes 'of'.
+
+ all of-valid x,y,z
+
+This is needed all of keys x, y and z.
+It is needed for these value of keys to be valid.
+
+If you need only 2 of these keys. you can write;
+
+ 2 of-valid x,y,z
+
+This is needed 2 of keys x, y or z.
+
+If you want valid values, use of-valid instead of 'of'.
 
 =item match
 
+This is regular expression.
+
+ key match ^[a-z]{2}\d{5}$
+
+If you want multiple regular expression.
+
+ key match ^[a-z]{2}\d{5}$, ^\d{5}[a-z]{1}\d{5}$, ...
+
+This is "or" condition. If value is match one of them, it is OK.
+
+=item re
+
+It is as same as 'match'.
+
+=item has
+
+ key has 3
+
+This means key has 3 values.
+
+If you want less than the number or grater than the number.
+You can write;
+
+ key has < 4
+ key has > 4
+
+=item eq
+
+ key eq STRING
+
+If key's value is as same as STRING, it is valid.
+
+=item ne
+
+ key ne STRING
+
+if key's value is NOT as same as STRING, it is valid.
+
+=item E<gt>, E<gt>=
+
+ key > 4
+
+If key's value is greater than number 4, it is valid.
+You can use '>=', too.
+
+If you want to check length of the value,
+put '~' before number as following.
+
+ key > ~ 4
+
+=item E<lt>, E<lt>=
+
+ key < 5
+
+If key's value is less than number 5, it is valid.
+You can use '<=', too.
+
+If you want to check length of the value,
+put '~' before number as following.
+
+ key < ~ 4
+
+=item between #,#
+
+ key between 3,5
+
+If key's value is in the range, it is valid.
+
+If you want to check length of the value,
+put '~' before number as following.
+
+ key between ~ 4,10
+
 =back
 
 =head1 HOW TO ADD OPERATOR
@@ -637,13 +982,16 @@ For example: is, are, match ...
 
 =item condition operator
 
-This is used in sentence.
+This is used in sentence only when Operator is 'is/are/isnt/arent'.
 
  Key Operator Condition
-              ~~~~~~~~~
-This is operator which is used for checking Value(s).
+     (is/are) ~~~~~~~~~
+   (isnt/arent)
 
-For example: num, alpha, alphanum ...
+This is operator which is used for checking Value(s).
+Operator should be 'is' or 'are'(thease are same) or 'isnt or arent'(thease are same).
+
+For example: num, alpha, alphanum, word ...
 
 =back
 
@@ -712,7 +1060,7 @@ Data::RuledValidator is made with plugins (from version 0.02).
 
 =head2 How to create plugins
 
-It's very easy. The name of the modules plugged in this is started from 'Data::RuledValidator::'.
+It's very easy. The name of the modules plugged in this is started from 'Data::RuledValidator::Plugin::'.
 
 for example:
 
@@ -744,6 +1092,41 @@ That's all. If you want to add normal_operator, use add_operator Class method.
  $valid = $validator_object;  # it is as same as $validator_object->valid;
  %valid = @$validator_object; # it is as same as %{$validator_object->result};
 
+=head1 INTERANL CLASS DATA
+
+It is just a memo.
+
+=over 4
+
+=item %RULE
+
+All rule for all object(which has different rule file).
+
+structure:
+
+ rule_name =>
+  {
+    _regex_group      => [],
+  $B!!(B# For group name, regexp can be used, for no need to find rule key is regexp or not,
+    # This exists.
+    id_key           => [],
+    # Rule has key which identify group name. this hash is {RULE_NAME => key_name}
+    # why array ref?
+    # for unique, we can set several key for id_key(it likes SQL unique)
+    coded_rule       => [],
+    # it is assemble of closure
+    time             => $time
+    # (stat 'rule_file')[9]
+  }
+
+=item %COND_OP
+
+The keys are condition operator names. The values is coderef(condition operator).
+
+=item %ML_CLOSSURE
+
+ {operator => sub{coderef which create closure} }
+
 =head1 NOTE
 
 Now, once rule is parsed, rule is change to code (assemble of closure) and
@@ -760,11 +1143,11 @@ store code to storable file. store code to shared memory.
 
 =over 4
 
-=item can 2 keys for id_key
+=item can take 2 keys for id_key
 
 =item More test
 
-I need more test.
+I need to do more test.
 
 =back
 
